/**
 * Copyright (C) 2010-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.terremark;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.dc.DataCenter;
import org.dasein.cloud.dc.DataCenterServices;
import org.dasein.cloud.dc.Region;
import org.jclouds.rest.RestContext;
import org.jclouds.vcloud.domain.VDC;
import org.jclouds.vcloud.terremark.TerremarkVCloudExpressAsyncClient;
import org.jclouds.vcloud.terremark.TerremarkVCloudExpressClient;

public class Vdc implements DataCenterServices {
    static public URI getVdcUri(RestContext<TerremarkVCloudExpressClient, TerremarkVCloudExpressAsyncClient> ctx, String id) {
        if( id.startsWith("http") ) {
            return URI.create(id);
        }
        return URI.create(ctx.getEndpoint() + "/v" + ctx.getApiVersion() + "/vdc/" + id);   
    }
    
    static public URI getVdcExtensionsUri(RestContext<TerremarkVCloudExpressClient, TerremarkVCloudExpressAsyncClient> ctx, String id) {
        if( id.startsWith("http") ) {
            return URI.create(id);
        }
        return URI.create(ctx.getEndpoint() + "/v" + ctx.getApiVersion() + "/extensions/vdc/" + id);   
    }
    
    private transient Terremark provider;
    
    Vdc(Terremark provider) {
        this.provider = provider;
    }
    
    @Override
    public DataCenter getDataCenter(String dataCenterId) throws InternalException, CloudException {
        VDC vdc = getVdc(dataCenterId);
        
        if( vdc != null ) {
            DataCenter dc = new DataCenter();
            
            dc.setActive(true);
            dc.setAvailable(true);
            dc.setName(vdc.getName());
            dc.setProviderDataCenterId(dataCenterId);
            dc.setRegionId(provider.getContext().getRegionId());
            return dc;
        }
        return null;
    }

    @Override
    public String getProviderTermForDataCenter(Locale locale) {
        return "zone";
    }

    @Override
    public String getProviderTermForRegion(Locale locale) {
        return "VDC";
    }

    @Override
    public Region getRegion(String regionId) throws InternalException, CloudException {
        if( regionId == null || !regionId.equals("us-miami") ) {
            return null;
        }
        Region region = new Region();
        
        region.setActive(true);
        region.setAvailable(true);
        region.setName("US/Miami");
        region.setProviderRegionId("us-miami");
        return region;
    }

    public VDC getVdc(String vdcId) {
        RestContext<TerremarkVCloudExpressClient, TerremarkVCloudExpressAsyncClient> ctx = provider.getCloudClient();

        try {
            VDC vdc = ctx.getApi().findVDCInOrgNamed(null, null);
        
            if( vdc == null ) {
                return null;
            }
            if( !Terremark.toId(vdc.getHref().toASCIIString()).equals(vdcId) ) {
                return null;
            }
            return vdc;
        }
        finally {
            ctx.close();
        }
    }
    
    @Override
    public Collection<DataCenter> listDataCenters(String regionId) throws InternalException, CloudException {
        ArrayList<DataCenter> dataCenters = new ArrayList<DataCenter>();
        
        if( regionId == null || !regionId.equals("us-miami") ) {
            return dataCenters;
        }
        RestContext<TerremarkVCloudExpressClient,TerremarkVCloudExpressAsyncClient> ctx = provider.getCloudClient();

        try {
            VDC vdc = ctx.getApi().findVDCInOrgNamed(null, null);
            DataCenter dc = new DataCenter();
            
            dc.setActive(true);
            dc.setAvailable(true);
            dc.setName(vdc.getName());
            dc.setRegionId("us-miami");
            dc.setProviderDataCenterId(Terremark.toId(vdc.getHref().toASCIIString()));
            dataCenters.add(dc);
        }
        finally {
            ctx.close();
        }
        return dataCenters;
    }

    @Override
    public Collection<Region> listRegions() throws InternalException, CloudException {
        ArrayList<Region> regions = new ArrayList<Region>();

        regions.add(getRegion("us-miami"));
        return regions;
    }
}
